(defun contour-dialog ()
(many-value-dialog "Type values for: mean1 mean2 sd1 sd2 rho n" 6 
    :length 40 :defaults '(0 0 1 1 .5 200)))

(defun contour-demo (&optional (numpts 30) (levels '(.1 .3 .5 .7 .9 .98)))
  (let* ((ans (contour-dialog))) 
   (if ans 
    (if (and (> (select ans 2) 0)
             (> (select ans 3) 0)
             (> (select ans 5) 0))
        (apply #'bi-norm (append ans (list numpts levels)))
        (block foo
         (message-dialog "sd1, sd2 and n must be positive")
         (contour-demo))))))


;bn-sample computes a sample of size n from a bivariate normal with 
;means mu1 and mu2, standard deviations s1 and s2 and correlation r.

(defun bn-sample (mu1 mu2 s1 s2 r n)
"Function args: mu1 mu2 s1 s2 r n
Returns a bivariate sample of size n from a normal distribution with mean
(mu1, mu2) variances s1 and s2, and covaraince r sqrt(s1*s2)."
   (let* ((z1 (normal-rand n))
          (z2 (normal-rand n))
          (x1 (+ mu1 (* s1 z1)))
          (fit (+ mu2 (* r s2 z1)))
          (x2 (+ fit (* (sqrt (- 1 (^ r 2))) s2 z2)))
         )
   (list x1 x2)))

;make-dens and make-dens-levels return functions that evaluate the 
;bivariate normal density in two ways.  In the first the argument is 
; the bivariate normal variable (x1, x2) itself.  In the second, the 
; argument is the chisq exponent of the bivariate normal.

   
(defun make-dens (mu1 mu2 s1 s2 r)
   (let* ((c1 (^ (* 2 (sqrt (- 1 (^ r 2))) pi s1 s2) -1))
          (c2 (* -1 (^ (* 2 (- 1 (^ r 2))) -1))))
        #'(lambda (x1 x2) (* c1 (exp (* c2 (+ 
                    (^ (/ (- x1 mu1) s1) 2) 
                    (^ (/ (- x2 mu2) s2) 2) 
                    (* -1 2 r (/ (- x1 mu1) s1) (/ (- x2 mu2) s2)))))))
   ))

(defun make-dens-level (mu1 mu2 s1 s2 r)
  (let* ((c1 (^ (* 2 (sqrt (- 1 (^ r 2))) pi s1 s2) -1))
        )
        #'(lambda (x) (* c1 (exp (* -.5 x))))
    ))

; bi-norm produces a 2D plot consisting of 
; 1. n observations from a bivariate normsl
; 2. level contours that mark a certain fraction of the probability 
; 3. levels which is a vector containing the probabilities for the contour
;    levels
; 4. numpts is the number of points at which the contours will be evaluated, 
;    as required by the contour-function.
 

(defun bi-norm (mu1 mu2 s1 s2 r n numpts levels &optional (dataset nil))
 (flet ((den (x1 x2) (funcall (make-dens mu1 mu2 s1 s2 r) x1 x2))
        (levl (x) (funcall (make-dens-level mu1 mu2 s1 s2 r) x)))
  (let* ((data (bn-sample mu1 mu2 s1 s2 r n))
         (plt (plot-points data :variable-labels '("x" "y") 
                :show nil
                :title (format nil
                "mu1=~a, mu2=~a, s1=~a, s2=~a, r=~a, n=~a Normal Contours Demo"
                mu1 mu2 s1 s2 r n))) )
    (send plt :finish-next-frame #'(lambda ()
             (send plt :add-function-contours 
               (function den) 
               (- mu1 (* 3 s1)) (+ mu1 (*  3 s1))
               (- mu2 (* 3 s2)) (+ mu2 (*  3 s2))
               (mapcar #'levl (chisq-quant levels 2))
               :num-points numpts
               :draw nil)
             #+msdos (send plt :redraw)))
    (send plt :contour-demo-controls mu1 mu2 s1 s2 r n)
    (mapcar #'funcall (send plt :slot-value 'finish-next-frame))
    (when dataset (rcode :data data :data-names (list "x1" "x2")))
    (mapcar #'(lambda (d s m) 
               (apply #'send plt :range d (+ m (list (* -3 s) (* 3 s)))))
               '(0 1) (list s1 s2) (list mu1 mu2))
    (send plt :redraw)
    (send plt :show-window)
    (setf plot plt)
    plt
    )))
    
(defmeth graph-proto :contour-demo-controls (&rest arg1)
   (send self :plot-controls)
   (let* ((plot self)
          (c1 (send checkbox-control-proto :new :title "New sample"
                :location (send self :locate-next-control)))
          (c2 (send checkbox-control-proto :new :title "New graph"
                :location (send self :locate-next-control))))
    (send self :add-overlay c1)
    (send self :add-overlay c2)
    (send plot :finish-next-frame #'(lambda ()
      (send (send plot :find-overlay 'smoother-proto) :redraw)
      (send (send plot :find-overlay 'curve-control-proto) :redraw)))
    (defmeth c1 :do-action (&rest args) 
       (send plot :start-buffering)
       (send plot :draw-next-frame '(0 1) (apply #'bn-sample arg1))
       (send plot :redraw-content)
       (send plot :buffer-to-screen)
       )
    (defmeth c2 :do-action (&rest args) (contour-demo))
    t))
;#|
(defun many-value-dialog (text num &key (len 25) (defaults (repeat nil num)))
"Function args: (text num &key (len 25) (defaults (repeat nil num))
Opens a dialog to get a list of numbers."
  (let* ((title (send text-item-proto :new text))
         (title1 (send text-item-proto :new (format nil
"Type ~a values separated by blanks" num)))
         (vals (send edit-text-item-proto :new "" :text-length len ))
         (get-answer #'(lambda () 
           (let* ((a (string-trim " " (send vals :text)))
                  (c1 (cond ((> (length a) 0)(select a 0)) (t nil)))
                  (ans (cond ((or (equal c1 #\') (equal c1 #\()) ;)
                              (with-input-from-string (s (send vals :text))
                              (eval (read s))))
                             ((equal a "") defaults)
                             (t (mapcar #'string-to-data 
                                    (divide-string (send vals :text)))))))
                (cond ((= (length ans) num) ans)
                      (t (message-dialog 
                         "Wrong number of arguments.  Try Again.")
                         (many-value-dialog text num 
                             :len len :defaults defaults))))))
         (cancel (send modal-button-proto :new "Cancel"))
         (done (send modal-button-proto :new "OK" :action 
                     #'(lambda () (funcall get-answer))))
         (dialog (send modal-dialog-proto :new
                      (list (list title)
                            (list title1)
                            (list vals)
                            (list done cancel)))))
        (send dialog :default-button done)
        (send dialog :modal-dialog) ))
;|#
(contour-demo)
